﻿local ADDONNAME, TrappersSenseData = ...
local VERSION = GetAddOnMetadata(ADDONNAME,"Version")
TrappersSenseData.Functions = {}
local TrappersSense = TrappersSenseData.Functions;

local _;--For good measure
-- Slash Commands
SLASH_TrappersSense1 = "/trapper";	-- /Commands

--[SVARS]Saved Variables, Per Character
local defaults  = {}
-- Text Color Constants
local CWhite  		= "|cffffffff";
local CRed    		= "|cffff0000";
local CGreen  		= "|cff00ff00";
local CBlue   		= "|cff6060ff";
local CCyan   		= "|cff00ffff";
local CMagenta		= "|cffff00ff";
local CYellow 		= "|cffffff00";
local CErrorRed		= "|cffff4040";
local CValidGreen	= "|cff40c840";
local RarityInfo = {
		-- _G["BATTLE_PET_BREED_QUALITY"..rarity]	ITEM_QUALITY_COLORS[rarity-1]
		[1] 		= {Desc = "Poor", 		Color = "|cff879d8a"},
		[2]			= {Desc = "Common", 	Color = "|cffffffff"},
		[3] 		= {Desc = "Uncommon", 	Color = "|cff00ff00"},
		[4] 		= {Desc = "Rare", 		Color = "|cff206edc"},
		[5] 		= {Desc = "Epic", 		Color = "|cffc030c0"},	--Likely Never Used
		[6] 		= {Desc = "Legendary",	Color = "|cffff8000"}	--Never Used: Just in case
}
--Frame Variables
local TrapFrame;

--List Variables
local PetList 		= {}
local Pets 			= {};
local PetListSize 	= 0;
local waitTimer 	= 0

function TrappersSenseData.Main()
	TrapFrame = CreateFrame("Frame", "TrapFrame", UIParent);
	TrapFrame:RegisterEvent("PET_JOURNAL_LIST_UPDATE");
	TrapFrame:RegisterEvent("PET_JOURNAL_PET_DELETED");
	--TrapFrame:RegisterEvent("CHAT_MSG_SYSTEM");
	TrapFrame:RegisterEvent("ADDON_LOADED");
	TrapFrame:SetScript("OnEvent", TrappersSense_OnEvent);
	TrapFrame:Hide();
end

function TrappersSense_OnEvent(self, event, ...)
	local addon = ...
	if (event == "ADDON_LOADED") then
		if ((...) == ADDONNAME) then
			--TrappersSense.Initialize(self)
			TrappersSense.LoadPets()
			DEFAULT_CHAT_FRAME:AddMessage(CGreen..ADDONNAME.." "..CRed..VERSION..CGreen..": Loaded. ")--Type "..Cyan..SLASH_TrappersSense1..Green.." for help.");
		end
	elseif (event == "PET_JOURNAL_LIST_UPDATE" or 
			event == "PET_JOURNAL_PET_DELETED") then
		if(waitTimer + 1 < time()) then --limit to once every second
			waitTimer = time();
			TrappersSense.LoadPets();
		else 
			return;
		end
	end
end

function SlashCmdList.TrappersSense(msg)
	print(msg)
	if msg == "run" then
		TrappersSense.LoadPets()
		return
	end
	if Pets[msg] == nil then 
		print("Empty. Exiting.")
		return 
	end;
	local id1, id2, id3, rar, num = Pets[msg][1],Pets[msg][2],Pets[msg][3],Pets[msg].TopRarity, Pets[msg].Count
	print(id1.."|n"..id2.."|n"..id3.."|n"..RarityInfo[rar].Color.._G["BATTLE_PET_BREED_QUALITY"..rar].."|r|n"..num)
end

function TrappersSense.LoadPets()
	-- Make sure all owned pets are shown in the Pet Journal
	C_PetJournal.ClearSearchFilter();
	C_PetJournal.SetAllPetSourcesChecked(true)
	C_PetJournal.SetAllPetTypesChecked(true)
	--PetList = {}
	Pets = {}
	local listCount, ownedCount = C_PetJournal.GetNumPets()
	--print(C_PetJournal.GetNumPets())
	for i = 1, listCount do
		local PID, sID, owned, Nick, Lv, fav, _, spec, tex, cat = C_PetJournal.GetPetInfoByIndex(i)
		--if (not owned) then break end; -- formality
		if Pets[spec] == nil then Pets[spec] = {[1]=0, [2]=0, [3]=0, ["TopRarity"]=1, ["Count"]=0, ["SID"] = sID} end -- new
		if owned then
			Pets[spec].Count = Pets[spec].Count + 1
			Pets[spec][Pets[spec].Count] = PID
			local _,_,_,_, petrarity = C_PetJournal.GetPetStats(PID)
			if petrarity > Pets[spec].TopRarity then Pets[spec].TopRarity = petrarity end
		end
	end
end

function TrappersSense.HookOnSight(self)
	local filePrefix, fileSuffix = "|TInterface\\PetBattles\\PetIcon-", ":16:16:0:0:128:256:63:103:129:169|t"
	local species, unit = self:GetUnit()
	
	--if Tooltip for non BattlePet, ignore
	if(not UnitIsWildBattlePet(unit)) then return end;
	
	--variables
	local counter = 0;
	local name;
	--Import tooltip
	local ToolTipText = {}
	for i = 1, select("#", self:GetRegions()) do
        local region = select(i, self:GetRegions())
        if (region and region:GetObjectType() == "FontString") then
            local text = region:GetText() -- string or nil
			table.insert(ToolTipText, text)
        end
    end
	--Get Target's Level
	local targetLevel = UnitBattlePetLevel(unit)
	--Get Pet Type
	local petType, fileName
	if ( targetLevel < 10 ) then
		petType = string.sub(ToolTipText[2], 13)
	else
		petType = string.sub(ToolTipText[2], 14)
	end
	--Modify type for file names
	if ( petType == "Dragonkin" ) then
		fileName = "Dragon";
	elseif ( petType == "Magic" ) then
		fileName = "Magical";
	elseif ( petType == "Aquatic" ) then
		fileName = "Water";
	else
		fileName = petType;
	end
	
	--Reset the tooltip Text
	self:ClearLines();
	local count = 0 
	if Pets[species] ~= nil then count = Pets[species].Count end
	self:AddLine(filePrefix..fileName..fileSuffix..CValidGreen.."["..targetLevel.."] "..ToolTipText[1]..(Pets[species]~=nil and " ("..Pets[species].Count.."/3)" or ""))
	--add known pet versions to tooltip
	if(Pets[species] ~= nil) and (Pets[species].Count ~= 0) then
		self:AddLine("Owned:");
		for i = 1, Pets[species].Count do	--List owned versions of the species
			--speciesID, customName, level, xp, maxXp, displayID, isFavorite, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable, unique, obtainable = C_PetJournal.GetPetInfoByPetID("petID")
			local _, nick, lv, _, _, _, isFavorite = C_PetJournal.GetPetInfoByPetID(Pets[species][i])
			--local PetID,_,_, nick, lv, isFavorite = C_PetJournal.GetPetInfoByIndex(Owned[i])
			local _,_,_,_, petrarity = C_PetJournal.GetPetStats(Pets[species][i])
			name = species;
			if (nick ~= nil) then
				name = nick --add nickname if has one
			end
			-- Add favorite icon
			local favIcon = "";
			if ( isFavorite ) then 
				favIcon = "|TInterface\\PetBattles\\PetJournal:12:12:0:0:512:1024:62:80:26:44|t"
			end
			-- Add level variance arrow
			local arrow = CWhite.."=|r "
			if (targetLevel > lv) then
				arrow = "|TInterface\\Minimap\\Minimap-PositionArrows:0:0:0:0:16:32:0:16:16:32|t"
			elseif (targetLevel < lv) then
				arrow = "|TInterface\\Minimap\\Minimap-PositionArrows:0:0:0:0:16:32:0:16:0:16|t"
			end
			self:AddLine(arrow..favIcon..name..", Lv"..lv..RarityInfo[petrarity].Color.." ("..RarityInfo[petrarity].Desc..")|r")
			
		end
	end
end

function TrappersSense.HookOnMinimap(self)
	if self:GetOwner()== Minimap then
		local petClaw = "|TInterface\\MINIMAP\\OBJECTICONS:12:12:0:0:256:256:97:128:129:160|t"
	
		local ToolTipText = GameTooltipTextLeft1:GetText()
		local ToolTip = {}
		--Break Apart the ToolTip Lines
		while string.len(ToolTipText) > 0  do
			local breakfind = string.find(ToolTipText, '\10')
			if (breakfind == nil ) then
				table.insert(ToolTip, ToolTipText)
				break
			else
				local line = string.sub(ToolTipText, 1, breakfind - 1)
				table.insert(ToolTip, line)
				ToolTipText = string.sub(ToolTipText,breakfind + 1)
			end
		end
		--Recreate Tooltip
		self:ClearLines()
		local newText = ""
		-- write new tooltip
		for i = 1, #ToolTip do
			if ToolTip[i] ~= nil then
				local chevron = ""
				local name = ToolTip[i]
				--Check for segments
				if string.find(ToolTip[i], "PositionArrows") ~= nil then
					chevron = string.sub(name, 0, string.find(name, "|t") + 1)
					name = string.sub(name, string.find(name, "|t") + 2)
				end
				if string.find(ToolTip[i], petClaw) ~= nil then
					name = string.sub(name, string.find(name, "|t") + 2)
				end
				-- reconstruct
				if Pets[name] ~= nil then ToolTip[i] = chevron..petClaw..name..RarityInfo[Pets[name].TopRarity].Color.." ("..Pets[name].Count.."/3)|r" end
				if (i ~= 1) then newText = newText.."\10" end
				newText = newText..ToolTip[i]
			end
		end
		self:SetText(newText)
	end
end

_G[ADDONNAME] = TrappersSense;
TrappersSenseData.Main()
GameTooltip:HookScript("OnTooltipSetUnit", TrappersSense.HookOnSight);
GameTooltip:HookScript("OnUpdate",TrappersSense.HookOnMinimap)


